<?php
class ControllerExtensionModuleFreeGift extends Controller {
	private $error = array();
	private $module = 'extension/module/free_gift';
	private $model_module = 'model_extension_module_free_gift';
	private $token = 'user_token';
	private $template = '';

	public function __construct($registry) {
		parent::__construct($registry);

		if (version_compare(VERSION , '2.3.0.2') <= 0) {
			$this->token = 'token';
			$this->template = '.tpl';
		}

		if (version_compare(VERSION , '2.2.0.0') <= 0) {
			$this->module = 'module/free_gift';
			$this->model_module = 'model_module_free_gift';
		}
	}

	public function index() {
		$this->load->language($this->module);

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model($this->module);

		$this->getList();
	}

	public function add() {
		$this->load->language($this->module);

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model($this->module);

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validateForm()) {
			$this->{$this->model_module}->addFreeGift($this->request->post);

			$this->session->data['success'] = $this->language->get('text_success');

			$url = '';

			if (isset($this->request->get['sort'])) {
				$url .= '&sort=' . $this->request->get['sort'];
			}

			if (isset($this->request->get['order'])) {
				$url .= '&order=' . $this->request->get['order'];
			}

			if (isset($this->request->get['page'])) {
				$url .= '&page=' . $this->request->get['page'];
			}

			$this->response->redirect($this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . $url, true));
		}

		$this->getForm();
	}

	public function edit() {
		$this->load->language($this->module);

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model($this->module);

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validateForm()) {
			$this->{$this->model_module}->editFreeGift($this->request->get['free_gift_id'], $this->request->post);

			$this->session->data['success'] = $this->language->get('text_success');

			$url = '';

			if (isset($this->request->get['sort'])) {
				$url .= '&sort=' . $this->request->get['sort'];
			}

			if (isset($this->request->get['order'])) {
				$url .= '&order=' . $this->request->get['order'];
			}

			if (isset($this->request->get['page'])) {
				$url .= '&page=' . $this->request->get['page'];
			}

			$this->response->redirect($this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . $url, true));
		}

		$this->getForm();
	}

	public function delete() {
		$this->load->language($this->module);

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model($this->module);

		if (isset($this->request->post['selected']) && $this->validateDelete()) {
			foreach ($this->request->post['selected'] as $free_gift_id) {
				$this->{$this->model_module}->deleteFreeGift($free_gift_id);
			}

			$this->session->data['success'] = $this->language->get('text_success');

			$url = '';

			if (isset($this->request->get['sort'])) {
				$url .= '&sort=' . $this->request->get['sort'];
			}

			if (isset($this->request->get['order'])) {
				$url .= '&order=' . $this->request->get['order'];
			}

			if (isset($this->request->get['page'])) {
				$url .= '&page=' . $this->request->get['page'];
			}

			$this->response->redirect($this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . $url, true));
		}

		$this->getList();
	}

	protected function getList() {
		if (isset($this->request->get['sort'])) {
			$sort = $this->request->get['sort'];
		} else {
			$sort = 'pd.name';
		}

		if (isset($this->request->get['order'])) {
			$order = $this->request->get['order'];
		} else {
			$order = 'ASC';
		}

		if (isset($this->request->get['page'])) {
			$page = $this->request->get['page'];
		} else {
			$page = 1;
		}

		$url = '';

		if (isset($this->request->get['sort'])) {
			$url .= '&sort=' . $this->request->get['sort'];
		}

		if (isset($this->request->get['order'])) {
			$url .= '&order=' . $this->request->get['order'];
		}

		if (isset($this->request->get['page'])) {
			$url .= '&page=' . $this->request->get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/dashboard', $this->token . '=' . $this->session->data[$this->token], true)
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('heading_title'),
			'href' => $this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . $url, true)
		);

		$data['add'] = $this->url->link($this->module . '/add', $this->token . '=' . $this->session->data[$this->token] . $url, true);
		$data['delete'] = $this->url->link($this->module . '/delete', $this->token . '=' . $this->session->data[$this->token] . $url, true);

		$data['free_gifts'] = array();

		$filter_data = array(
			'sort'  => $sort,
			'order' => $order,
			'start' => ($page - 1) * $this->config->get('config_limit_admin'),
			'limit' => $this->config->get('config_limit_admin')
		);

		$free_gift_total = $this->{$this->model_module}->getTotalFreeGifts();

		$results = $this->{$this->model_module}->getFreeGifts($filter_data);

		foreach ($results as $result) {
			$data['free_gifts'][] = array(
				'free_gift_id' => $result['free_gift_id'],
				'name'         => $result['name'],
				'min_amount'   => $result['min_amount'],
				'status'       => $result['status'],
				'edit'         => $this->url->link($this->module . '/edit', $this->token . '=' . $this->session->data[$this->token] . '&free_gift_id=' . $result['free_gift_id'] . $url, true)
			);
		}

		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_list'] = $this->language->get('text_list');
		$data['text_no_results'] = $this->language->get('text_no_results');
		$data['text_confirm'] = $this->language->get('text_confirm');
		$data['text_enabled'] = $this->language->get('text_enabled');
		$data['text_disabled'] = $this->language->get('text_disabled');

		$data['column_name'] = $this->language->get('column_name');
		$data['column_status'] = $this->language->get('column_status');
		$data['column_purchase'] = $this->language->get('column_purchase');
		$data['column_action'] = $this->language->get('column_action');

		$data['button_add'] = $this->language->get('button_add');
		$data['button_edit'] = $this->language->get('button_edit');
		$data['button_delete'] = $this->language->get('button_delete');

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];

			unset($this->session->data['success']);
		} else {
			$data['success'] = '';
		}

		if (isset($this->request->post['selected'])) {
			$data['selected'] = (array)$this->request->post['selected'];
		} else {
			$data['selected'] = array();
		}

		$url = '';

		if ($order == 'ASC') {
			$url .= '&order=DESC';
		} else {
			$url .= '&order=ASC';
		}

		if (isset($this->request->get['page'])) {
			$url .= '&page=' . $this->request->get['page'];
		}

		$data['sort_name'] = $this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . '&sort=pd.name' . $url, true);
		$data['sort_status'] = $this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . '&sort=fg.status' . $url, true);

		$url = '';

		if (isset($this->request->get['sort'])) {
			$url .= '&sort=' . $this->request->get['sort'];
		}

		if (isset($this->request->get['order'])) {
			$url .= '&order=' . $this->request->get['order'];
		}

		$pagination = new Pagination();
		$pagination->total = $free_gift_total;
		$pagination->page = $page;
		$pagination->limit = $this->config->get('config_limit_admin');
		$pagination->url = $this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . $url . '&page={page}', true);

		$data['pagination'] = $pagination->render();

		$data['results'] = sprintf($this->language->get('text_pagination'), ($free_gift_total) ? (($page - 1) * $this->config->get('config_limit_admin')) + 1 : 0, ((($page - 1) * $this->config->get('config_limit_admin')) > ($free_gift_total - $this->config->get('config_limit_admin'))) ? $free_gift_total : ((($page - 1) * $this->config->get('config_limit_admin')) + $this->config->get('config_limit_admin')), $free_gift_total, ceil($free_gift_total / $this->config->get('config_limit_admin')));

		$data['sort'] = $sort;
		$data['order'] = $order;

		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view($this->module . '_list' . $this->template, $data));
	}

	protected function getForm() {
		$data['text_form'] = !isset($this->request->get['free_gift_id']) ? $this->language->get('text_add') : $this->language->get('text_edit');

		$data['heading_title'] = $this->language->get('heading_title');
		
		$data['text_yes'] = $this->language->get('text_yes');
		$data['text_no'] = $this->language->get('text_no');
		$data['text_enabled'] = $this->language->get('text_enabled');
		$data['text_disabled'] = $this->language->get('text_disabled');

		$data['entry_name'] = $this->language->get('entry_name');
		$data['entry_min'] = $this->language->get('entry_min');
		$data['entry_delivery'] = $this->language->get('entry_delivery');
		$data['entry_group'] = $this->language->get('entry_group');
		$data['entry_status'] = $this->language->get('entry_status');

		$data['button_save'] = $this->language->get('button_save');
		$data['button_cancel'] = $this->language->get('button_cancel');

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset($this->error['name'])) {
			$data['error_name'] = $this->error['name'];
		} else {
			$data['error_name'] = array();
		}

		if (isset($this->error['amount'])) {
			$data['error_amount'] = $this->error['amount'];
		} else {
			$data['error_amount'] = array();
		}

		if (isset($this->error['group'])) {
			$data['error_group'] = $this->error['group'];
		} else {
			$data['error_group'] = array();
		}

		$url = '';

		if (isset($this->request->get['sort'])) {
			$url .= '&sort=' . $this->request->get['sort'];
		}

		if (isset($this->request->get['order'])) {
			$url .= '&order=' . $this->request->get['order'];
		}

		if (isset($this->request->get['page'])) {
			$url .= '&page=' . $this->request->get['page'];
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/dashboard', $this->token . '=' . $this->session->data[$this->token], true)
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('heading_title'),
			'href' => $this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . $url, true)
		);

		if (!isset($this->request->get['free_gift_id'])) {
			$data['action'] = $this->url->link($this->module . '/add', $this->token . '=' . $this->session->data[$this->token] . $url, true);
		} else {
			$data['action'] = $this->url->link($this->module . '/edit', $this->token . '=' . $this->session->data[$this->token] . '&free_gift_id=' . $this->request->get['free_gift_id'] . $url, true);
		}

		$data['cancel'] = $this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token] . $url, true);

		if (isset($this->request->get['free_gift_id']) && ($this->request->server['REQUEST_METHOD'] != 'POST')) {
			$free_gift_info = $this->{$this->model_module}->getFreeGift($this->request->get['free_gift_id']);
		}

		$data['user_token'] = $this->session->data[$this->token];
		$data['module_path'] = $this->module;

		$this->load->model('customer/customer_group');

		$data['customer_groups'] = $this->model_customer_customer_group->getCustomerGroups();

		if (isset($this->request->post['product_name'])) {
			$data['product_name'] = $this->request->post['product_name'];
		} elseif (isset($this->request->get['free_gift_id'])) {
			$this->load->model('catalog/product');
			$product = $this->model_catalog_product->getProduct($free_gift_info['product_id']);
			$data['product_name'] = $product['name'];
		} else {
			$data['product_name'] = '';
		}

		if (isset($this->request->post['product_id'])) {
			$data['product_id'] = $this->request->post['product_id'];
		} elseif (!empty($free_gift_info)) {
			$data['product_id'] = $free_gift_info['product_id'];
		} else {
			$data['product_id'] = '';
		}

		if (isset($this->request->post['min_amount'])) {
			$data['min_amount'] = $this->request->post['min_amount'];
		} elseif (!empty($free_gift_info)) {
			$data['min_amount'] = $free_gift_info['min_amount'];
		} else {
			$data['min_amount'] = '';
		}

		if (isset($this->request->post['customer_group_id'])) {
			$data['customer_group_id'] = (array)$this->request->post['customer_group_id'];
		} elseif (!empty($free_gift_info)) {
			$customer_group_id = explode(",",$free_gift_info['customer_group_id']);
			$data['customer_group_id'] = $customer_group_id;
		} else {
			$data['customer_group_id'] = array();
		}

		if (isset($this->request->post['free_delivery'])) {
			$data['free_delivery'] = $this->request->post['free_delivery'];
		} elseif (!empty($free_gift_info)) {
			$data['free_delivery'] = $free_gift_info['free_delivery'];
		} else {
			$data['free_delivery'] = '';
		}

		if (isset($this->request->post['status'])) {
			$data['status'] = $this->request->post['status'];
		} elseif (!empty($free_gift_info)) {
			$data['status'] = $free_gift_info['status'];
		} else {
			$data['status'] = '';
		}

		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view($this->module . '_form' . $this->template, $data));
	}

	protected function validateForm() {
		if (!$this->user->hasPermission('modify', $this->module)) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		if (empty($this->request->post['product_name'])) {
			$this->error['name'] = $this->language->get('error_name');
		}

		if (empty($this->request->post['min_amount'])) {
			$this->error['amount'] = $this->language->get('error_amount');
		}

		if (empty($this->request->post['customer_group_id'])) {
			$this->error['group'] = $this->language->get('error_group');
		}

		if (empty($this->request->post['product_id'])) {
			$this->error['warning'] = $this->language->get('error_product_id');
		}

		$this->load->model('catalog/product');
		$product = $this->model_catalog_product->getProduct($this->request->post['product_id']);

		if ($product['name'] != $this->request->post['product_name']) {
			$this->error['warning'] = $this->language->get('error_product_id');
		}

		$this->load->model($this->module);

		if (isset($this->request->get['free_gift_id'])) {
			$free_gift_id = $this->request->get['free_gift_id'];
		} else {
			$free_gift_id = 0;
		}

		$result = $this->{$this->model_module}->checkFreeGiftDuplicate($this->request->post['customer_group_id'], $this->request->post['min_amount'], $free_gift_id);

		if ($result && $this->request->post['status'] == 1) {
			$this->error['warning'] = $this->language->get('error_duplicate');
		}

		return !$this->error;
	}

	protected function validateDelete() {
		if (!$this->user->hasPermission('modify', $this->module)) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		return !$this->error;
	}

	public function autocomplete() {
		$json = array();

		if (isset($this->request->get['filter_name'])) {
			$this->load->model($this->module);

			if (isset($this->request->get['filter_name'])) {
				$filter_name = $this->request->get['filter_name'];
			} else {
				$filter_name = '';
			}

			$filter_data = array(
				'filter_name'  => $filter_name,
				'start'        => 0,
				'limit'        => 10
			);

			$results = $this->{$this->model_module}->getProducts($filter_data);

			foreach ($results as $result) {
				$json[] = array(
					'product_id' => $result['product_id'],
					'name'       => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8')),
					'model'      => $result['model'],
					'price'      => $result['price']
				);
			}
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}

	public function install() {
		$this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "free_gift` (
				`free_gift_id` int(11) NOT NULL AUTO_INCREMENT,
				`min_amount` int(11) NOT NULL,
				`product_id` int(11) NOT NULL,
				`customer_group_id` text NOT NULL,
				`free_delivery` tinyint(1) NOT NULL,
				`status` tinyint(1) NOT NULL,
				PRIMARY KEY (`free_gift_id`)
			) ENGINE=MyISAM DEFAULT CHARSET=utf8;");

		$query = $this->db->query("SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = '" . DB_DATABASE . "' AND TABLE_NAME = '" . DB_PREFIX . "product' AND COLUMN_NAME = 'product_type'");
		if (!$query->num_rows) {
			$this->db->query("ALTER TABLE " . DB_PREFIX . "product ADD product_type tinyint(1) NOT NULL DEFAULT 0");
		}

		$this->load->model('setting/setting');
		$free_gift['module_free_gift_status'] = 1;
		$this->model_setting_setting->editSetting('module_free_gift', $free_gift);
	}

	public function uninstall() {
		$this->load->model('setting/setting');
		$this->model_setting_setting->deleteSetting('module_free_gift');
	}
}
