<?php
class ControllerExtensionTotalFreeGift extends Controller {
	private $error = array();
	private $module = 'extension/total/free_gift';
	private $token = 'user_token';
	private $link_extension = 'marketplace/extension';
	private $template = '';
	private $setting_key = 'total_free_gift';

	public function __construct($registry) {
		parent::__construct($registry);

		if (version_compare(VERSION , '2.3.0.2') <= 0) {
			$this->token = 'token';
			$this->template = '.tpl';
			$this->link_extension = 'extension/extension';
			$this->setting_key = 'free_gift';
		}

		if (version_compare(VERSION , '2.2.0.0') <= 0) {
			$this->module = 'total/free_gift';
			$this->link_extension = 'extension/total';
		}
	}

	public function index() {
		$this->load->language($this->module);

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model('setting/setting');

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
			$this->model_setting_setting->editSetting($this->setting_key, $this->request->post);

			$this->session->data['success'] = $this->language->get('text_success');

			$this->response->redirect($this->url->link($this->link_extension, $this->token . '=' . $this->session->data[$this->token] . '&type=total', true));
		}

		$data['heading_title'] = $this->language->get('heading_title');
		
		$data['text_edit'] = $this->language->get('text_edit');
		$data['text_enabled'] = $this->language->get('text_enabled');
		$data['text_disabled'] = $this->language->get('text_disabled');

		$data['entry_status'] = $this->language->get('entry_status');
		$data['entry_sort_order'] = $this->language->get('entry_sort_order');

		$data['button_save'] = $this->language->get('button_save');
		$data['button_cancel'] = $this->language->get('button_cancel');

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/dashboard', $this->token . '=' . $this->session->data[$this->token], true)
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_extension'),
			'href' => $this->url->link($this->link_extension, $this->token . '=' . $this->session->data[$this->token] . '&type=total', true)
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('heading_title'),
			'href' => $this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token], true)
		);

		$data['action'] = $this->url->link($this->module, $this->token . '=' . $this->session->data[$this->token], true);

		$data['cancel'] = $this->url->link($this->link_extension, $this->token . '=' . $this->session->data[$this->token] . '&type=total', true);

		$data['status'] = $this->setting_key . '_status';
		$data['sort_order'] = $this->setting_key . '_sort_order';

		if (isset($this->request->post[$this->setting_key . '_status'])) {
			$data['total_free_gift_status'] = $this->request->post[$this->setting_key . '_status'];
		} else {
			$data['total_free_gift_status'] = $this->config->get($this->setting_key . '_status');
		}

		if (isset($this->request->post[$this->setting_key . '_sort_order'])) {
			$data['total_free_gift_sort_order'] = $this->request->post[$this->setting_key . '_sort_order'];
		} else {
			$data['total_free_gift_sort_order'] = $this->config->get($this->setting_key . '_sort_order');
		}

		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view($this->module . $this->template, $data));
	}

	protected function validate() {
		if (!$this->user->hasPermission('modify', $this->module)) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		return !$this->error;
	}
}